import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { Icon, Form, Modal, Tag, message } from 'antd'
import { EditableTag, JsonFormItems } from 'components'
import { validateDescriptor as vd } from 'utils'
import styles from './AlarmSetting.less'

const FormItem = Form.Item

class AlarmSetting extends React.PureComponent {
  constructor (props) {
    super(props)
    this.state = {
      inputVisible: false,
      newReceiverVerified: true,
      receiversVerifiedList: [],
    }

    this.onOk = this.onOk.bind(this)
    this.onCancel = this.onCancel.bind(this)
    this.handleTagInputChange = this.handleTagInputChange.bind(this)
    this.handleTagClose = this.handleTagClose.bind(this)
    this.handleButtonClick = this.handleButtonClick.bind(this)
  }
  // Modal event
  onOk () {
    let submit = () => {
      const data = {
        ...this.props.form.getFieldsValue(),
      }
      data.receiver = this.props.formModel.receiver

      this.props.dispatch({ type: 'cluster/updateAlarm', payload: { ...data } }).then((resp) => {
        message.success(resp.info, 5)
        this.onCancel()
      })
    }
    if (this.props.formModel.flag) {
      this.props.form.validateFields((errors) => {
        let receiverLen = this.props.formModel.receiver.length
        let receiversVerified = this.state.receiversVerifiedList.indexOf(false) === -1
        if (errors) {
          return
        }
        if (!receiversVerified) {
          return
        }
        if (receiverLen === 0) {
          this.setState({ newReceiverVerified: false })
          return
        }
        submit()
      })
    } else {
      submit()
    }
  }
  onCancel () {
    this.props.dispatch({ type: 'cluster/updateState', payload: { alarmSettingModalVisible: false } })
    this.props.form.resetFields()
    this.setState({ receiversVerifiedList: [] })
  }
  onAddEmail () {
    let formModel = JSON.parse(JSON.stringify(this.props.formModel))

    formModel.receiver.push('')

    this.props.dispatch({ type: 'cluster/updateState', payload: { alarmSettingModel: formModel } })
  }
  onTagInputChange (verified, prevValue, currentValue) {
    let formModel = JSON.parse(JSON.stringify(this.props.formModel))
    let target = formModel.receiver.indexOf(prevValue)

    if (verified) {
      formModel.receiver[target] = currentValue
      this.props.dispatch({ type: 'cluster/updateState', payload: { alarmSettingModel: formModel } })
    }
  }
  onTagClose (value) {
    let formModel = JSON.parse(JSON.stringify(this.props.formModel))
    let target = formModel.receiver.indexOf(value)

    formModel.receiver.splice(target, 1)
    this.props.dispatch({ type: 'cluster/updateState', payload: { alarmSettingModel: formModel } })
  }
  getFormDataModel () {
    return {
      flag: {
        label: '报警开关',
        type: 'Switch',
        initialValue: this.props.formModel.flag,
        props: {
          defaultChecked: this.props.formModel.flag,
          onChange: (value) => {
            this.props.formModel.flag = value
          },
        },
      },
      smtp: {
        label: 'SMTP服务器',
        type: 'Input',
        initialValue: this.props.formModel.smtp,
        rules: this.props.formModel.flag ? [vd.required, vd.smtp] : [{}],
        props: {
          disabled: !this.props.formModel.flag,
        },
      },
      port: {
        label: '端口',
        type: 'InputNumber',
        initialValue: this.props.formModel.port,
        props: {
          min: 0,
          max: 65535,
          disabled: !this.props.formModel.flag,
        },
        rules: this.props.formModel.flag ? [{
          required: true, message: '请输入0~65535的整数',
        }] : [{}],
      },
      sender: {
        label: '发件人邮箱',
        type: 'Input',
        initialValue: this.props.formModel.sender,
        rules: this.props.formModel.flag ? [vd.required, vd.email] : [{}],
        props: {
          disabled: !this.props.formModel.flag,
        },
      },
      password: {
        label: '密码',
        type: 'Input',
        props: {
          type: 'password',
          disabled: !this.props.formModel.flag,
        },
        initialValue: '',
        rules: this.props.formModel.flag ? [vd.required] : [{}],
      },
    }
  }
  // new email event
  handleButtonClick () {
    if (this.props.formModel.flag) {
      this.setState({ newReceiverVerified: true })
      if (this.state.receiversVerifiedList.indexOf(false) > -1) {
        return
      }
      this.onAddEmail()
    }
  }
  handleTagInputChange (verified, preValue, currentValue) {
    let target = this.props.formModel.receiver.indexOf(preValue)
    let receiversVerifiedList = [].concat(this.state.receiversVerifiedList)

    receiversVerifiedList[target] = verified
    this.setState({ receiversVerifiedList })

    this.onTagInputChange(verified, preValue, currentValue)
  }
  handleTagClose (value) {
    let target = this.props.formModel.receiver.indexOf(value)
    let receiversVerifiedList = [].concat(this.state.receiversVerifiedList)

    receiversVerifiedList.splice(target, 1)
    this.setState({ receiversVerifiedList })

    this.onTagClose(value)
  }
  render () {
    const { inputVisible, newReceiverVerified } = this.state
    const { confirmLoading, formModel } = this.props
    if (!this.props.modalVisible || !formModel) {
      return null
    }
    const modalProps = {
      onOk: this.onOk,
      onCancel: this.onCancel,
      visible: this.props.modalVisible,
      maskClosable: false,
      title: '报警配置',
      style: { top: 47 },
      confirmLoading,
    }
    const jsonFormItemProps = {
      dataModel: this.getFormDataModel(),
      getFieldDecorator: this.props.form.getFieldDecorator,
    }
    const formItemProps = {
      label: <span className="ant-form-item-required">收件人邮箱</span>,
      key: 'receiver',
      labelCol: { span: 6 },
      wrapperCol: { span: 14 },
    }

    return (
      <Modal {...modalProps}>
        <Form>
          <JsonFormItems {...jsonFormItemProps} />
          <FormItem {...formItemProps}>
            {
              this.props.formModel.receiver.map((email) => {
                let tagProps = {
                  value: email,
                  key: email,
                  tagMaxLen: 20,
                  onTagChange: this.handleTagInputChange,
                  onTagClose: this.handleTagClose,
                  disabled: !this.props.formModel.flag,
                  editable: email === '',
                }
                return (
                  <EditableTag {...tagProps} />
                )
              })
            }

            {!inputVisible && (
              <div className={styles.addTag}>
                <Tag className={this.props.formModel.flag ? '' : styles.addTagDisabled} onClick={this.handleButtonClick}>
                  <Icon type="plus" /> 新增邮箱
                </Tag>
                { this.props.formModel.flag && !newReceiverVerified && <div className={styles.invalid}>至少新增一个收件人邮箱</div>}
              </div>
            )}
          </FormItem>
        </Form>
      </Modal>
    )
  }
}

AlarmSetting.propTypes = {
  dispatch: PropTypes.func,
  modalVisible: PropTypes.bool,
  formModel: PropTypes.object,
  confirmLoading: PropTypes.bool,
}

const mapStateToProps = ({ cluster, loading }) => {
  if (!cluster.alarmSettingModalVisible) {
    return {}
  }
  return {
    modalVisible: cluster.alarmSettingModalVisible,
    formModel: cluster.alarmSettingModel,
    confirmLoading: loading.effects['cluster/updateAlarm'],
  }
}

export default connect(mapStateToProps)(Form.create()(AlarmSetting))
