import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { Button, Icon } from 'antd'
import { JsonFormModal } from '../JsonForm'

const dataTransform = (data) => {
  let commandsChildren = []

  for (let key of Object.keys(data)) {
    commandsChildren.push({
      label: data[key],
      value: key,
    })
  }

  return commandsChildren
}

class ClusterOprtInfo extends React.PureComponent {
  constructor (props) {
    super(props)
    this.state = {
      optResult: '',
    }

    this.onCancel = this.onCancel.bind(this)
  }

  onCancel () {
    let dispatch = this.props.dispatch
    dispatch({ type: 'cluster/updateState', payload: { commandModalVisible: false } })
    this.setState({
      optResult: '',
    })
  }

  render () {
    let { visible, commands, confirmLoading, dispatch } = this.props
    if (!commands) {
      return null
    }

    const DataModel = {
      operation: {
        label: '集群操作',
        type: 'Select',
        props: {
          placeholder: '请选择操作命令',
          onChange: (value) => {
            this.setState({
              optResult: '',
            })
            dispatch({ type: 'cluster/operateCommand', payload: { cmd_type: value } }).then((resp) => {
              this.setState({
                optResult: resp.data,
              })
            })
          },
        },
        children: dataTransform(commands),
      },
      info: {
        label: <span>操作结果&nbsp;
          { confirmLoading ? <Icon type="loading" /> : '' }
        </span>,
        type: 'TextArea',
        initialValue: this.state.optResult,
        props: {
          rows: 15,
          style: {
            color: 'inherit',
            background: '#fefefe',
            cursor: 'default',
          },
          disabled: true,
        },
      },
    }

    let modalOpts = {
      onCancel: this.onCancel.bind(this),
      dataModel: DataModel,
      visible,
      maskClosable: false,
      title: '集群快捷操作',
      style: { top: 47 },
      width: 750,
      formProps: {
        layout: 'vertical',
      },
      formItemProps: null,
      footer: [
        <Button key="back" onClick={this.onCancel}>关闭</Button>,
      ],
    }
    return (
      <JsonFormModal {...modalOpts} />
    )
  }
}

ClusterOprtInfo.propTypes = {
  dispatch: PropTypes.func,
  visible: PropTypes.bool,
  formDataModel: PropTypes.object,
  confirmLoading: PropTypes.bool,
}

const mapStateToProps = ({ cluster, loading }) => {
  if (!cluster.commandModalVisible) {
    return {}
  }
  return {
    visible: cluster.commandModalVisible,
    commands: cluster.commands,
    confirmLoading: loading.effects['cluster/operateCommand'],
  }
}

export default connect(mapStateToProps)(ClusterOprtInfo)
