import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { message, Tooltip, Icon } from 'antd'
import { JsonFormModal } from '../JsonForm'

const dataTypeandPropsTransform = (data) => {
  let dataType = data.content.type
  let otherPropsModel = {}
  switch (dataType) {
    case 'integer':
      otherPropsModel = {
        props: { min: -1000000, max: 1000000 },
      }
      break
    case 'enum':
      otherPropsModel = {
        children: data.content.values.map((key) => {
          return {
            label: key,
            value: key,
          }
        }),
      }
      break
    case 'boolean':
    case 'string':
      break
    default:
      throw new Error(`Unknowning parameter type: ${dataType} in clusterSettingModel`)
  }
  return otherPropsModel
}
const dataTransform = (data) => {
  let dataModel = {}

  for (let key of Object.keys(data)) {
    let param = data[key]
    dataModel[key] = {
      label: <span>
        {param.name}&nbsp;
        {
          param.longdesc &&
          <Tooltip title={param.longdesc}>
            <Icon type="question-circle-o" />
          </Tooltip>
        }
      </span>,
      initialValue: param.value,
      type: param.content.type,
      ...dataTypeandPropsTransform(param),
    }
  }
  return dataModel
}
class ClusterSetting extends React.PureComponent {
  onOk (data, resetFields) {
    let dispatch = this.props.dispatch
    dispatch({ type: 'cluster/update', payload: { ...data } }).then((resp) => {
      message.success(resp.info, 5)
      dispatch({ type: 'cluster/updateState', payload: { clusterSettingModalVisible: false } })
      resetFields()
    })
  }
  onCancel () {
    let dispatch = this.props.dispatch
    dispatch({ type: 'cluster/updateState', payload: { clusterSettingModalVisible: false } })
  }

  render () {
    let { visible, formDataModel, confirmLoading } = this.props
    if (!formDataModel) {
      return null
    }

    formDataModel = dataTransform(formDataModel)

    let modalOpts = {
      onOk: this.onOk.bind(this),
      onCancel: this.onCancel.bind(this),
      dataModel: formDataModel,
      visible,
      maskClosable: false,
      title: '首选项配置',
      style: { top: 47 },
      formItemProps: {
        labelCol: {
          span: 11,
        },
        wrapperCol: {
          span: 9,
        },
        style: {
          marginBottom: 2,
        },
      },
      confirmLoading,
    }
    return (
      <JsonFormModal {...modalOpts} />
    )
  }
}

ClusterSetting.propTypes = {
  dispatch: PropTypes.func,
  visible: PropTypes.bool,
  formDataModel: PropTypes.object,
  confirmLoading: PropTypes.bool,
}

const mapStateToProps = ({ cluster, loading }) => {
  if (!cluster.clusterSettingModalVisible) {
    return {}
  }
  return {
    visible: cluster.clusterSettingModalVisible,
    formDataModel: cluster.clusterSettingModel,
    confirmLoading: loading.effects['cluster/update'],
  }
}

export default connect(mapStateToProps)(ClusterSetting)
