import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { message } from 'antd'
import { JsonFormModal } from 'components'
import { validateDescriptor as vd } from 'utils'

class GfsSetting extends React.PureComponent {
  constructor (props) {
    super(props)

    this.onOk = this.onOk.bind(this)
    this.onCancel = this.onCancel.bind(this)
  }
  onOk (data) {
    let submitUrl = data.enabled ? 'cluster/updateGfs' : 'cluster/deleteGfs'
    delete data.enabled
    this.props.dispatch({ type: submitUrl, payload: { ...data } }).then((resp) => {
      message.success(resp.info)
      this.props.dispatch({ type: 'resource/query' })
      this.props.dispatch({ type: 'cluster/updateState', payload: { gfsModalVisible: false } })
    })
  }
  onCancel () {
    this.props.dispatch({ type: 'cluster/updateState', payload: { gfsModalVisible: false } })
  }

  getFormDataModel () {
    let { formModel, dispatch } = this.props

    return {
      enabled: {
        label: '启用GFS',
        type: 'Switch',
        initialValue: formModel.enabled,
        props: {
          defaultChecked: formModel.enabled,
          onChange: (value) => {
            dispatch({ type: 'cluster/updateState', payload: { gfsModel: { ...formModel, enabled: value } } })
          },
        },
      },
      device: {
        label: '共享磁盘路径',
        type: 'Input',
        initialValue: formModel.device,
        rules: formModel.enabled ? [vd.required, vd.devPath, vd.validPath] : [{}],
        props: {
          disabled: !formModel.enabled,
        },
      },
      mount_dir: {
        label: '挂载目录',
        type: 'Input',
        initialValue: formModel.mount_dir,
        rules: formModel.enabled ? [vd.required, vd.absPath, vd.validPath] : [{}],
        props: {
          disabled: !formModel.enabled,
        },
      },
      max_num: {
        label: '最大GFS节点数',
        type: 'InputNumber',
        initialValue: String(formModel.max_num || 2),
        rules: formModel.enabled ? [vd.required] : [{}],
        props: {
          disabled: !formModel.enabled,
        },
      },
    }
  }
  render () {
    const { confirmLoading, formModel, visible } = this.props
    if (!visible || !formModel) {
      return null
    }
    let modalOpts = {
      onOk: this.onOk,
      onCancel: this.onCancel,
      dataModel: this.getFormDataModel(formModel),
      visible,
      maskClosable: false,
      title: 'GFS配置',
      style: { top: 47 },
      formItemProps: {
        labelCol: {
          span: 8,
        },
        wrapperCol: {
          span: 12,
        },

      },
      confirmLoading,
    }

    return (
      <JsonFormModal {...modalOpts} />
    )
  }
}

GfsSetting.propTypes = {
  dispatch: PropTypes.func,
  visible: PropTypes.bool,
  formModel: PropTypes.object,
  confirmLoading: PropTypes.bool,
}

const mapStateToProps = ({ cluster, loading }) => {
  if (!cluster.gfsModalVisible) {
    return {}
  }
  return {
    visible: cluster.gfsModalVisible,
    formModel: cluster.gfsModel,
    confirmLoading: loading.effects['cluster/updateGfs'] || loading.effects['cluster/deleteGfs'],
  }
}

export default connect(mapStateToProps)(GfsSetting)
