import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { Form, Modal, Tabs, message } from 'antd'
import { JsonFormItems } from 'components'
import { validateDescriptor } from 'utils'
import { featureSwitch } from 'config'

const TabPane = Tabs.TabPane
const { diskHbMode, ucHbMode } = featureSwitch

class HeartBeatSetting extends React.PureComponent {
  constructor (props) {
    super(props)
    this.state = {
      tabSeletedKey: '0',
      confirmVisible: false,
    }

    this.handleTabChange = this.handleTabChange.bind(this)
    this.onOkSettingModel = this.onOkSettingModel.bind(this)
    this.onCancelSettingModal = this.onCancelSettingModal.bind(this)
  }
  // Modal event
  onOkSettingModel () {
    const validate = (tabKey) => {
      this.validateTabContent(tabKey).then(() => {
        tabKey += 1
        if (tabKey < 3) {
          validate(tabKey)
        } else { // 所有tab页验证通过后，调出输入密码对话框
          const data = {
            ...this.props.form.getFieldsValue(),
          }
          const submitData = this.trandferSubmitData(data)
          this.setState({ nodeAuthVisible: false })
          this.props.dispatch({ type: 'cluster/updateHeartBeat', payload: { ...submitData } }).then((resp) => {
            message.success(`${resp.info}, 您需要重启每个节点使之生效。`, 5)
            this.onCancelSettingModal()
          })
        }
      }).catch(() => {
        return false
      })
    }

    validate(0)
  }

  onCancelSettingModal () {
    this.props.dispatch({ type: 'cluster/updateState', payload: { heartBeatModalVisible: false } })
    this.props.form.resetFields()
    this.setState({ tabSeletedKey: '0' })
  }

  getNhbFormModel (tabKey) {
    const { formModel } = this.props
    let nhbFormDataModel = {}
    let bcConfig = {}
    let mcConfig = {}
    let nodeList = {}
    let children = [{
      label: '广播',
      value: 'bc',
    }, {
      label: '多播',
      value: 'mc',
    }]

    if (ucHbMode) {
      children.push({
        label: '单播',
        value: 'uc',
      })
    }

    const getNodeListItem = (nodeName, nodeIp, nodeIndex) => {
      // 规避nodeName中的特殊字符，用索引值作为对象key值
      nodeList[`${tabKey}.nodeList.${nodeIndex}`] = {
        label: `节点心跳IP-${nodeName}`,
        type: 'Input',
        initialValue: nodeIp,
        rules: formModel[tabKey].flag ? [validateDescriptor.ip, validateDescriptor.required] : [{}],
        props: {
          disabled: !formModel[tabKey].flag,
        },
      }
    }

    if (!formModel[tabKey]) {
      formModel[tabKey] = tabKey === 0 ? { type: 'bc', flag: true } : { type: formModel[0].type, flag: false }
    }
    if (tabKey === 0) {
      nhbFormDataModel[`${tabKey}.type`] = {
        label: '心跳类型',
        type: 'RadioGroup',
        props: {
          onChange: (e) => {
            formModel[tabKey].type = e.target.value
          },
        },
        initialValue: formModel[tabKey].type,
        children,
      }
    } else { // 备用心跳可选项由主心跳决定
      formModel[tabKey].type = formModel[0].type
      nhbFormDataModel[`${tabKey}.flag`] = {
        label: '开启备用心跳',
        type: 'Switch',
        initialValue: formModel[tabKey].flag,
        props: {
          defaultChecked: formModel[tabKey].flag,
          onChange: (value) => {
            formModel[tabKey].flag = value
            // 清除验证状态
            this.props.form.resetFields([`${tabKey}.bindnetaddr`, `${tabKey}.mcastport`, `${tabKey}.mcastaddr`])
            this.props.nodeList.forEach((node, index) => {
              this.props.form.resetFields([`${tabKey}.nodeList.${index}`])
            })
          },
        },
      }
    }

    bcConfig[`${tabKey}.bindnetaddr`] = {
      label: '绑定网段',
      type: 'Input',
      initialValue: formModel[tabKey].bindnetaddr ? formModel[tabKey].bindnetaddr : '',
      rules: formModel[tabKey].flag ? [validateDescriptor.netSegment, validateDescriptor.required] : [{}],
      props: {
        disabled: !formModel[tabKey].flag,
      },
    }

    bcConfig[`${tabKey}.mcastport`] = {
      label: '端口',
      type: 'InputNumber',
      initialValue: formModel[tabKey].mcastport ? formModel[tabKey].mcastport : '',
      props: {
        min: 1025,
        max: 65535,
        disabled: !formModel[tabKey].flag,
      },
      rules: formModel[tabKey].flag ? [{
        required: true, message: '请输入1025~65535之间的UDP端口。',
      }] : [{}],
    }
    mcConfig[`${tabKey}.mcastaddr`] = {
      label: '多播地址',
      type: 'Input',
      initialValue: formModel[tabKey].mcastaddr ? formModel[tabKey].mcastaddr : '',
      rules: formModel[tabKey].flag ? [validateDescriptor.multiCastIp, validateDescriptor.required] : [{}],
      props: {
        disabled: !formModel[tabKey].flag,
      },
    }
    // 当前心跳配置有nodelist，读取配置本身的信息
    if (formModel[tabKey].hbaddrs && formModel[tabKey].hbaddrs.length > 0) { // 兼容只配一路nodelist的情况
      formModel[tabKey].hbaddrs.forEach((node, index) =>
        getNodeListItem(node.nodeid, node.ip, index)
      )
    } else if (!formModel[tabKey].hbaddrs && formModel[0].hbaddrs) {
      // 当前心跳配置没有nodelist，主心跳配置有nodelist，读取主心跳的nodelist，ip需要重新配置
      formModel[0].hbaddrs.forEach((node, index) =>
        getNodeListItem(node.nodeid, '', index)
      )
    } else {
      // 无心跳配置时，读取节点列表作为nodelist的数据源，ip需要重新配置
      this.props.nodeList.forEach((node, index) =>
        getNodeListItem(node.id, '', index)
      )
    }

    switch (formModel[tabKey].type) {
      case 'mc':
        return {
          ...nhbFormDataModel,
          ...bcConfig,
          ...mcConfig,
          ...nodeList,
        }
      default:
        return {
          ...nhbFormDataModel,
          ...bcConfig,
          ...nodeList,
        }
    }
  }

  getDhbFormModel (tabKey) {
    const { formModel, devices } = this.props
    let dhbConfig = {}
    let intervalChildren = []
    if (!formModel[tabKey]) {
      formModel[tabKey] = { flag: false }
    }
    for (let i = 5; i > 0;) {
      intervalChildren.push({
        label: `${i}s`,
        value: i,
      })
      i -= 1
    }

    dhbConfig[`${tabKey}.flag`] = {
      label: '开启磁盘心跳',
      type: 'Switch',
      initialValue: formModel[tabKey].flag,
      props: {
        defaultChecked: formModel[tabKey].flag,
        onChange: (value) => {
          formModel[tabKey].flag = value
          this.props.form.resetFields([`${tabKey}.disk`, `${tabKey}.label`, `${tabKey}.interval`])
        },
      },
    }

    dhbConfig[`${tabKey}.disk`] = {
      label: '设备',
      type: 'Select',
      children: devices,
      props: {
        placeholder: '请选择',
        disabled: !formModel[tabKey].flag,
        showSearch: true,
        allowClear: true,
        filterOption: (input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0,
      },
      initialValue: formModel[tabKey] ? formModel[tabKey].disk : '',
      rules: formModel[2].flag ? [validateDescriptor.required] : [{}],
    }

    dhbConfig[`${tabKey}.label`] = {
      label: '磁盘标签',
      type: 'Input',
      initialValue: formModel[tabKey] ? formModel[tabKey].label : '',
      rules: formModel[2].flag ? [validateDescriptor.required, validateDescriptor.alphanumeric, validateDescriptor.length(4, 20)] : [{}],
      props: {
        disabled: !formModel[tabKey].flag,
      },
    }

    dhbConfig[`${tabKey}.interval`] = {
      label: '间隔',
      type: 'Select',
      children: intervalChildren,
      initialValue: formModel[tabKey] ? formModel[tabKey].interval : '',
      rules: formModel[2].flag ? [validateDescriptor.required] : [{}],
      props: {
        placeholder: '请选择',
        disabled: !formModel[tabKey].flag,
      },
    }

    return dhbConfig
  }
  // 验证每个tab内容的输入合法性
  validateTabContent (tabKey) {
    let oldTabKey = this.state.tabSeletedKey
    let validateItems = [`${tabKey}.bindnetaddr`, `${tabKey}.mcastport`]
    if (this.props.formModel[0].type === 'mc') {
      validateItems.push(`${tabKey}.mcastaddr`)
    }

    if (this.props.formModel[0] && this.props.formModel[0].hbaddrs) {
      this.props.formModel[0].hbaddrs.forEach((node, index) =>
        validateItems.push(`${tabKey}.nodeList.${index}`)
      )
    } else {
      this.props.nodeList.forEach((node, index) =>
        validateItems.push(`${tabKey}.nodeList.${index}`)
      )
    }

    if (parseInt(tabKey, 10) === 2) {
      validateItems = [`${tabKey}.disk`, `${tabKey}.label`, `${tabKey}.interval`]
    }
    // 配合单页验证，手动切换tab，原因：主心跳类型变化时，会引发备用心跳输入项改变
    this.setState({ tabSeletedKey: `${tabKey}` })

    return new Promise((resolve, reject) => {
      if (!this.props.formModel[tabKey].flag) { // 当该页开关关闭时，不对该页内容进行验证，切回提交页
        this.setState({ tabSeletedKey: `${oldTabKey}` })
        return resolve()
      }
      return this.props.form.validateFields(validateItems, (errors) => {
        if (errors) { // 该页验证不通过时，停留在本页
          reject()
        } else { // 该页验证通过时，停留在当前页
          this.setState({ tabSeletedKey: `${oldTabKey}` })
          resolve()
        }
      })
    })
  }

  // 切换tab
  handleTabChange (tabKey) {
    this.validateTabContent(this.state.tabSeletedKey).then(() => {
      this.setState({ tabSeletedKey: tabKey })
    }).catch(() => {
      return false
    })
  }
  trandferSubmitData (data) {
    const getNodeName = (number, nodeIndex) => {
      if (this.props.formModel[number] && this.props.formModel[number].hbaddrs) {
        return this.props.formModel[number].hbaddrs[nodeIndex].nodeid
      } else if (this.props.formModel[0] && this.props.formModel[0].hbaddrs) {
        return this.props.formModel[0].hbaddrs[nodeIndex].nodeid
      }

      return this.props.nodeList[nodeIndex].id
    }
    let submitModel = {
      hbinfo: [],
    }
    // 注意：form数据中，主心跳没有flag项，备用心跳没有type项
    for (let number of Object.keys(data)) {
      if (parseInt(number, 10) === 0) {
        data[number].flag = true
      }

      if (!data[number].flag) { // 未开启的配置项，不进行提交
        continue
      }
      // 非多播网络心跳，主备心跳删除mcastaddr项
      if (data[number].type !== 'mc' && (parseInt(number, 10) === 1 && !data[0].mcastaddr)) {
        delete data[number].mcastaddr
      }

      // 当主心跳类型为广播时，主心跳与备用心跳的提交数据中需加入broadcast: 'yes'
      if (data[number].type === 'bc' || (parseInt(number, 10) === 1 && data[0].broadcast)) {
        data[number].broadcast = 'yes'
      }

      if (data[number].mcastport && !data[number].hbaddrs) {
        data[number].hbaddrs = Object.keys(data[number].nodeList).map((nodeIndex) => {
          return {
            ip: data[number].nodeList[nodeIndex],
            nodeid: getNodeName(number, nodeIndex),
          }
        })
      }
      delete data[number].nodeList
      delete data[number].type
      delete data[number].flag
      submitModel.hbinfo.push(data[number])
    }
    submitModel.hbinfo.forEach((info, index) => {
      info.ringnumber = index
    })

    return submitModel
  }
  render () {
    const { tabSeletedKey } = this.state
    if (!this.props.modalVisible || !this.props.formModel) {
      return null
    }
    const modalProps = {
      onOk: this.onOkSettingModel,
      onCancel: this.onCancelSettingModal,
      visible: this.props.modalVisible,
      maskClosable: false,
      title: '心跳配置',
      width: 550,
      style: { top: 47 },
      confirmLoading: this.props.confirmLoading,
    }

    const nhbFormItemProps = (tabKey) => {
      return {
        dataModel: this.getNhbFormModel(tabKey),
        getFieldDecorator: this.props.form.getFieldDecorator,
        formItemProps: {
          labelCol: { span: 8 },
          wrapperCol: { span: 12 },
        },
      }
    }

    const dhbFormItemProps = {
      dataModel: this.getDhbFormModel(2),
      getFieldDecorator: this.props.form.getFieldDecorator,
      formItemProps: {
        labelCol: { span: 8 },
        wrapperCol: { span: 12 },
      },
    }

    return (
      <div>
        <Modal {...modalProps}>
          <Form>
            <Tabs defaultActiveKey="0" activeKey={tabSeletedKey} onChange={this.handleTabChange}>
              <TabPane tab="主心跳" key="0">
                <JsonFormItems {...nhbFormItemProps(0)} />
              </TabPane>
              <TabPane tab="备用心跳" key="1">
                <JsonFormItems {...nhbFormItemProps(1)} />
              </TabPane>
              { diskHbMode && <TabPane tab="磁盘心跳" key="2">
                <JsonFormItems {...dhbFormItemProps} />
              </TabPane>}
            </Tabs>
          </Form>
        </Modal>
      </div>
    )
  }
}

HeartBeatSetting.propTypes = {
  dispatch: PropTypes.func,
  modalVisible: PropTypes.bool,
  formModel: PropTypes.array,
  confirmLoading: PropTypes.bool,
  nodeList: PropTypes.array,
  devices: PropTypes.array,
}

const mapStateToProps = ({ cluster, loading, node }) => {
  if (!cluster.heartBeatModalVisible) {
    return {}
  }
  return {
    modalVisible: cluster.heartBeatModalVisible,
    formModel: cluster.heartBeatModel,
    nodeList: node.list,
    devices: cluster.devices,
    confirmLoading: loading.effects['cluster/updateHeartBeat'],
  }
}

export default connect(mapStateToProps)(Form.create()(HeartBeatSetting))
