import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { message, Modal } from 'antd'
import { JsonFormModal } from 'components'
import { validateDescriptor as vd } from 'utils'

class ScriptGenerator extends React.PureComponent {
  constructor (props) {
    super(props)

    this.onOk = this.onOk.bind(this)
    this.onCancel = this.onCancel.bind(this)
  }
  onOk (data) {
    Modal.confirm({
      title: '确认生成脚本',
      content: `如果已经有脚本名为：${data.name}，该脚本将会被覆盖， 确认生成脚本吗？`,
      onOk: () => {
        return this.props.dispatch({ type: 'cluster/createScript', payload: { ...data } }).then((resp) => {
          message.success(resp.info)
          this.props.dispatch({ type: 'cluster/updateState', payload: { scriptModalVisible: false } })
        })
      },
    })
  }
  onCancel () {
    this.props.dispatch({ type: 'cluster/updateState', payload: { scriptModalVisible: false } })
  }

  render () {
    const { loading, visible } = this.props
    if (!visible) {
      return null
    }
    const dataModel = {
      name: {
        label: '脚本名称',
        type: 'Input',
        rules: [vd.required],
      },
      start: {
        label: '启动脚本命令',
        type: 'Input',
        rules: [vd.required],
      },
      stop: {
        label: '停止脚本命令',
        type: 'Input',
        rules: [vd.required],
      },
      monitor: {
        label: '监控进程',
        type: 'TextArea',
        rules: [vd.required],
        props: {
          placeholder: '在监控进程列表中，每一行代表一个进程',
          rows: 5,
        },
      },
    }
    let modalOpts = {
      onOk: this.onOk,
      onCancel: this.onCancel,
      dataModel,
      visible,
      maskClosable: false,
      title: '脚本生成器',
      style: { top: 47 },
      formItemProps: {
        labelCol: {
          span: 8,
        },
        wrapperCol: {
          span: 12,
        },
      },
      loading,
    }

    return (
      <JsonFormModal {...modalOpts} />
    )
  }
}

ScriptGenerator.propTypes = {
  dispatch: PropTypes.func,
  visible: PropTypes.bool,
  loading: PropTypes.bool,
}

const mapStateToProps = ({ cluster, loading }) => {
  if (!cluster.scriptModalVisible) {
    return {}
  }
  return {
    visible: cluster.scriptModalVisible,
    loading: loading.effects['cluster/updateScript'],
  }
}

export default connect(mapStateToProps)(ScriptGenerator)
