import React from 'react'
import PropTypes from 'prop-types'
import { Tag, Input, Tooltip } from 'antd'
import AsyncValidator from 'async-validator'
import { validateDescriptor } from 'utils'
import styles from './index.less'

class EditableTag extends React.Component {
  constructor (props) {
    super(props)
    this.state = {
      tagValue: this.props.value,
      // tagEditable: false,
      tagEditable: this.props.editable,
      inputTipVisible: false,
    }

    this.changeInputState = this.changeInputState.bind(this)
    this.handleClick = this.handleClick.bind(this)
  }

  changeInputState (e) {
    const descriptor = {
      email: validateDescriptor.email,
      required: validateDescriptor.required,
    }
    const validator = new AsyncValidator(descriptor)
    const value = e.target.value
    validator.validate({ email: value, required: value }, (errors) => {
      if (errors) {
        this.props.onTagChange(false, this.state.tagValue)
        this.setState({ inputTipVisible: true })
        return
      }
      this.props.onTagChange(true, this.state.tagValue, value)
      this.setState({ tagValue: value, tagEditable: false, inputTipVisible: false })
    })
  }
  handleClose () {
    this.props.onTagClose(this.state.tagValue)
  }
  // tag event
  handleClick (e) {
    const tagName = e.target.tagName
    if (tagName !== 'I') {
      this.setState({ tagEditable: true })
    }
  }

  render () {
    const { tagValue, tagEditable, inputTipVisible } = this.state
    const isLongTag = tagValue.length > this.props.tagMaxLen
    let tagContent = isLongTag ? `${tagValue.slice(0, this.props.tagMaxLen)}...` : tagValue
    let eidtableTag = (
      <div>
        <Tag key={tagValue} closable afterClose={() => this.handleClose()} onClick={this.handleClick} className={tagEditable ? styles.editableTag : styles.staticTag}>
          {
            tagEditable ?
              <Input size="large" defaultValue={tagValue} onPressEnter={this.changeInputState} onBlur={this.changeInputState} placeholder={this.props.placeholder || '请填写'} onClick={(e) => { e.stopPropagation() }} autoFocus />
              :
              tagContent
          }
        </Tag>
        {tagEditable && inputTipVisible && <div className={styles.invalid}>{validateDescriptor.email.message}</div>}
      </div>
    )
    let staticTag = (
      <Tag key={tagValue} className={styles.staticTag}>
        { isLongTag ? `${tagValue.slice(0, this.props.tagMaxLen)}...` : tagValue }
      </Tag>
    )

    let tagElem = this.props.disabled ? staticTag : eidtableTag

    return isLongTag ? <Tooltip title={tagValue} key={tagValue}>{tagElem}</Tooltip> : tagElem
  }
}

EditableTag.propTypes = {
  value: PropTypes.string, // 标签的默认值
  tagMaxLen: PropTypes.number, // 标签内显示文字的最大长度
  onTagChange: PropTypes.func, // (verified, preValue, currentValue)
  onTagClose: PropTypes.func, // (value)
  disabled: PropTypes.bool, // 标签是否可编辑
  editable: PropTypes.bool,
  placeholder: PropTypes.string,
}

export default EditableTag
