import React from 'react'
import PropTypes from 'prop-types'
import { Form, Radio, Select, Input, InputNumber, Cascader, Switch, Modal } from 'antd'

const FormItem = Form.Item
const RadioGroup = Radio.Group
const Option = Select.Option

/*
dataModel: {
    //对象key即提交数据项的key值
    name: {
      //表单项名称
      label: str || reactNode,
      //指定表单项的组件类型，与antd组件相对应，包括
      //Radio Select or 或者其他的reactNode
      type: str || reactNode
      //初始值
      initialValue: ''
      //用于表单项验证，写法与form表单项的验证规则一致
      rules: [{
        type: 'email', message: 'The input is not valid E-mail!',
      }, {
        required: true, message: 'Please input your E-mail!',
      }],
      //指定select 和 radiogroup里的内容
      children: [{
        label: '',
        value: '',
      }],
      //表单项组件的附属属性
      props: {},
    },
}
*/
const JsonFormItems = ({
  getFieldDecorator,
  dataModel,
  formItemProps = {
    labelCol: {
      span: 6,
    },
    wrapperCol: {
      span: 14,
    },
  },
}) => {
  const assembleTag = (type, props, children) => {
    let Type = type
    switch (type) {
      case 'Select':
      case 'enum':
        return (
          <Select {...props}>
            {
              children.map(c =>
                <Option key={c.value} value={c.value}>{c.label}</Option>
              )
            }
          </Select>
        )
      case 'RadioGroup':
        return (
          <RadioGroup {...props}>
            {
              children.map(c =>
                <Radio key={c.value} value={c.value}>{c.label}</Radio>
              )
            }
          </RadioGroup>
        )
      case 'Input':
      case 'string':
        return (<Input {...props} />)
      case 'InputNumber':
      case 'integer':
      case 'int':
        return (<InputNumber {...props} />)
      case 'Cascader':
        return (<Cascader {...props} />)
      case 'Switch':
      case 'bool':
      case 'boolean':
        return (<Switch {...props} />)
      case 'TextArea':
        return (<Input.TextArea {...props} />)
      default:
        return typeof type === 'function' ? (<Type {...props} />) : type
    }
  }


  return (
    <div>
      {
        Object.keys(dataModel).map((key) => {
          let v = dataModel[key]
          return (
            <FormItem label={v.label} key={key} style={{ marginBottom: 8 }} {...formItemProps}>
              {
                getFieldDecorator(key, {
                  initialValue: v.initialValue,
                  rules: v.rules,
                  valuePropName: v.type === 'boolean' ? 'checked' : 'value', // FIXME: that's bad~
                })(assembleTag(v.type, v.props, v.children))
              }
            </FormItem>
          )
        })
      }
    </div>
  )
}

const _JsonFormModal = ({
  form: {
    getFieldDecorator,
    validateFields,
    getFieldsValue,
    resetFields,
  },
  formProps = {
    layout: 'horizontal',
  },
  dataModel,
  onOk,
  onCancel,
  formItemProps,
  ...otherProps
}) => {
  const onOkWrapper = () => {
    validateFields((errors) => {
      if (errors) {
        return
      }
      const data = {
        ...getFieldsValue(),
      }
      onOk(data, resetFields)
    })
  }
  const onCancelWrapper = () => {
    onCancel()
    resetFields()
  }
  let modalProps = {
    onOk: onOkWrapper,
    onCancel: onCancelWrapper,
    ...otherProps,
  }
  return (
    <Modal {...modalProps}>
      <Form {...formProps}>
        <JsonFormItems {...{ dataModel, getFieldDecorator, formItemProps }} />
      </Form>
    </Modal>
  )
}

const JsonFormModal = Form.create()(_JsonFormModal)

JsonFormItems.propTypes = {
  dataModel: PropTypes.object, // 标签的默认值
  formItemProps: PropTypes.object,
}

_JsonFormModal.propTypes = {
  dataModel: PropTypes.object,
  formProps: PropTypes.object,
  formItemProps: PropTypes.object,
  onOk: PropTypes.func,
  onCancel: PropTypes.func,
}

export default { JsonFormItems, JsonFormModal }
