/* global window */
import React from 'react'
import PropTypes from 'prop-types'
import { Icon, Popover, Layout, Button, Menu, Dropdown, Tooltip } from 'antd'
import classnames from 'classnames'
import { prefix, featureSwitch } from 'config'
import styles from './Header.less'
import Menus from './Menu'

const { licenseMode } = featureSwitch
const Header = ({ switchSider, siderFold, isNavbar, menuPopoverVisible, license, showLicense, location, switchMenuPopover, clickNavMenu, isLicenseLoading, clickTopMenu, hbstatus }) => {
  const menusProps = {
    siderFold: false,
    isNavbar,
    handleClickNavMenu: clickNavMenu,
    location,
  }
  const userMenu = (
    <Menu onClick={clickTopMenu} style={{ top: 5 }}>
      <Menu.Item key="setting" className={styles.userMenuItem}>刷新设置</Menu.Item>
      <Menu.Item key="logout" className={styles.userMenuItem}>退出登录</Menu.Item>
    </Menu>
  )
  let licenseDom = license ?
    (<span><Button className={(license.version_id === 0 && license.halicense_is_expired === 1) || license.type === '未激活' ? styles.redTwinkling : ''} onClick={showLicense}>{ isLicenseLoading ? <Icon type="loading" /> : (license.type || '未知版本') }</Button><div className={styles.dividerVertical} /></span>) :
    (<Icon type="loading" />)
  return (
    <Layout.Header className={styles.header}>
      {isNavbar
        ? <Popover placement="bottomLeft" onVisibleChange={switchMenuPopover} visible={menuPopoverVisible} overlayClassName={styles.popovermenu} trigger="click" content={<Menus {...menusProps} />}>
          <div className={styles.button}>
            <Icon type="bars" />
          </div>
        </Popover>
        : <div
          className={styles.button}
          onClick={switchSider}
        >
          <Icon type={classnames({ 'menu-unfold': siderFold, 'menu-fold': !siderFold })} />
        </div>}
      <div className={styles.rightWarpper}>

        {
          Boolean(hbstatus) &&
          <Tooltip placement="bottom" title="心跳连接发生故障，请联系技术支持">
            <Icon type="heartbeat" className={styles.redTwinkling} />
          </Tooltip>
        }
        { licenseMode ? licenseDom : '' }
        <Dropdown overlay={userMenu}>
          <Button className={styles.userNameBtn}><Icon type="user" style={{ fontSize: 17 }} />{ window.localStorage.getItem(`${prefix}username`) }<Icon type="down" /></Button>
        </Dropdown>
      </div>
    </Layout.Header>
  )
}

Header.propTypes = {
  user: PropTypes.object,
  logout: PropTypes.func,
  switchSider: PropTypes.func,
  siderFold: PropTypes.bool,
  isNavbar: PropTypes.bool,
  menuPopoverVisible: PropTypes.bool,
  location: PropTypes.object,
  switchMenuPopover: PropTypes.func,
  license: PropTypes.object,
  showLicense: PropTypes.func,
  clickNavMenu: PropTypes.func,
  clickTopMenu: PropTypes.func,
}

export default Header
