import React from 'react'
import PropTypes from 'prop-types'
import { Menu, Icon } from 'antd'
import { featureSwitch } from 'config'
import { Link } from 'react-router-dom'

const { gfsMode, alarmMode, scriptGtMode, logMode, clusterOptMode, toolMode, drbdMode } = featureSwitch
const Menus = ({ siderFold, handleClickNavMenu }) => {
  let defaultSelectedKeys = ['resource']
  return (
    <Menu
      mode="inline"
      theme="dark"
      onClick={handleClickNavMenu}
      selectedKeys={defaultSelectedKeys}
      defaultOpenKeys={siderFold ? [] : ['haCluster', 'tool']}
      inlineCollapsed={siderFold}
    >
      <Menu.Item key="resource">
        <Link to="/resource">
          <Icon type="home" />
          <span>系统</span>
        </Link>
      </Menu.Item>

      <Menu.SubMenu key="haCluster" title={<span><Icon type="setting" /><span>集群配置</span></span>}>
        <Menu.Item key="cluterSetting">首选项配置</Menu.Item>
        <Menu.Item key="heartBeatSetting">心跳配置</Menu.Item>
        { alarmMode ? <Menu.Item key="alarmSetting">报警配置</Menu.Item> : ''}
        { gfsMode ? <Menu.Item key="GfsSetting">GFS配置</Menu.Item> : ''}
        { drbdMode ? <Menu.Item key="DrbdSetting">DRBD配置</Menu.Item> : ''}
      </Menu.SubMenu>
      {
        toolMode ?
          <Menu.SubMenu key="tool" title={<span><Icon type="tool" /><span>工具</span></span>}>
            { logMode ? <Menu.Item key="logDownload">日志下载</Menu.Item> : ''}
            { scriptGtMode ? <Menu.Item key="scriptGenerator">脚本生成器</Menu.Item> : ''}
            { clusterOptMode ? <Menu.Item key="clusterOprtInfo">集群快捷操作</Menu.Item> : ''}
          </Menu.SubMenu> :
          ''
      }
    </Menu>
  )
}

Menus.propTypes = {
  siderFold: PropTypes.bool,
  handleClickNavMenu: PropTypes.func,
}

export default Menus
