/* global window, FileReader, document, Blob, navigator */
import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { Icon, Modal, message, Upload, Button, Form, Row, Col } from 'antd'
import { JsonFormItems } from 'components'
import { config, getLicenseType } from 'utils'

class LicenseView extends React.PureComponent {
  constructor (props) {
    super(props)
    this.state = {
      fileList: null,
    }

    this.handleCancel = this.handleCancel.bind(this)
    this.handleDownload = this.handleDownload.bind(this)
  }

  getLicenseInfoFormData () {
    let licenseInfo = {
      type: {
        label: '当前版本',
        type: <span className="ant-form-text">{this.props.license.type || '未知版本'}</span>,
      },
      feedback: {
        label: '机器码',
        type: <span className="ant-form-text">{this.props.license.feedback}</span>,
      },
    }
    // 正式版显示注册时间，不显示过期时间
    if (this.props.license.type === '正式版') {
      return {
        ...licenseInfo,
        register_time: {
          label: '注册时间',
          type: <span className="ant-form-text">{this.props.license.register_time}</span>,
        },
      }
    }
    if (this.props.license.type !== '未激活' && this.props.license.type) {
      // 非正式版显示过期时间，不显示注册时间
      return {
        ...licenseInfo,
        valid_period: {
          label: '过期时间',
          type: <span className="ant-form-text">{this.props.license.valid_period}</span>,
        },
      }
    }

    return { ...licenseInfo }
  }
  handleCancel () {
    this.props.dispatch({ type: 'app/updateState', payload: { licenseModalVisible: false } })
  }
  handleDownload () {
    let fileObj = this.props.license.feedback
    let blob = new Blob([fileObj], {
      type: 'application/x-vv',
    })
    // 适用于IE10+
    if (window.navigator.msSaveOrOpenBlob) {
      navigator.msSaveBlob(blob, 'feedback.txt')
      return
    }

    let aLink = document.createElement('a')
    let evt = document.createEvent('MouseEvents')
    evt.initEvent('click', false, true)
    aLink.download = 'feedback.txt'
    aLink.href = window.URL.createObjectURL(blob)
    aLink.dispatchEvent(evt)
  }
  render () {
    const dispatch = this.props.dispatch
    const context = this
    if (!this.props.modalVisible) {
      return null
    }
    const modalProps = {
      visible: this.props.modalVisible,
      maskClosable: false,
      title: '版本信息',
      style: { top: 47 },
      onCancel: this.handleCancel,
      footer: [
        <Button key="back" onClick={this.handleCancel}>关闭</Button>,
      ],
    }
    const uploadProps = {
      name: 'file',
      action: config.api.license,
      accept: '.dat',
      multiple: false,
      beforeUpload (file) {
        const isLt1M = file.size / 1024 / 1024 < 1
        if (!isLt1M) {
          message.error('文件必须小于 1MB!')
          this.setState({ fileList: null })
        }
        return isLt1M
      },
      customRequest (file) {
        let reader = new FileReader()
        const fileObj = file.file

        reader.readAsText(fileObj, 'UTF-8')
        reader.onload = function (e) {
          let fileText = e.target.result
          dispatch({ type: 'app/registerLicense', payload: { file: fileText } }).then((resp) => {
            fileObj.status = 'done'
            const fileList = [file.file]
            const licenseModel = getLicenseType(resp.data)
            context.setState({ fileList })
            message.success(resp.message, 5)

            context.props.dispatch({ type: 'login/updateLicense', payload: { license: licenseModel } })
          }).catch(() => {
            fileObj.status = 'error'
            const fileList = [file.file]
            context.setState({ fileList })
          })
        }
      },
      onChange (info) {
        context.setState({ fileList: info.fileList })
      },
      fileList: this.state.fileList,
    }

    const JsonFormItemsProps = {
      dataModel: this.getLicenseInfoFormData(),
      getFieldDecorator: this.props.form.getFieldDecorator,
      formItemProps: {
        labelCol: { span: 6 },
        wrapperCol: { span: 14 },
      },
    }

    const optBtns = (<Row>
      <Col span={2} key="placeholder" />
      <Col span={9} key="downloadBtn">
        <Button onClick={this.handleDownload}>
          <Icon type="download" /> 导出注册文件
        </Button>
      </Col>
      <Col span={9} key="uploadBtn">
        <Upload {...uploadProps} >
          <Button>
            <Icon type="upload" /> 导入授权文件
          </Button>
        </Upload>
      </Col>
    </Row>)

    return (
      <Modal {...modalProps}>
        <Form layout="horizontal">
          <JsonFormItems {...JsonFormItemsProps} />
        </Form>
        { this.props.license.type !== '正式版' && optBtns }

      </Modal>
    )
  }
}

LicenseView.propTypes = {
  modalVisible: PropTypes.bool,
  license: PropTypes.object,
  dispatch: PropTypes.func,
}

const mapStateToProps = ({ app, login }) => {
  if (!app.licenseModalVisible) {
    return {}
  }
  return {
    modalVisible: app.licenseModalVisible,
    license: login.license,
  }
}

export default connect(mapStateToProps)(Form.create()(LicenseView))
