import React from 'react'
import PropTypes from 'prop-types'
import { Button, Icon, Card, Badge } from 'antd'
import { queryArray, handleOperation } from 'utils'
import { connect } from 'dva'

class NodeView extends React.PureComponent {
  onClick (id, action, data) {
    return handleOperation(this.props.dispatch, 'node', id, action, data).then(() => {
      this.props.dispatch({ type: 'resource/query' })
    })
  }
  render () {
    let { node, confirmLoading } = this.props

    if (!node) {
      return null
    }

    const header = (
      <span title={node.id}>
        {
          confirmLoading ?
            <Icon type="loading" style={{ marginRight: 8 }} /> :
            <Badge status={node.running ? 'success' : 'error'} />
        }
        { node.id }
      </span>
    )

    return (
      <Card title={header}>
        { !node.running ?
          <Button onClick={() => this.onClick(node.id, 'start')}>
            <Icon type="caret-right" />启动
          </Button> :
          <Button onClick={() => this.onClick(node.id, 'stop')}>
            <Icon type="stop" />停止
          </Button>
        }
        { !node.standby ?
          <Button onClick={() => this.onClick(node.id, 'standby')} style={{ marginTop: 8 }}>
            <Icon type="down-circle" />备用
          </Button> :
          <Button onClick={() => this.onClick(node.id, 'unstandby')} style={{ marginTop: 8 }}>
            <Icon type="up-circle" />启用
          </Button>
        }

      </Card>
    )
  }
}

NodeView.propTypes = {
  node: PropTypes.object,
  dispatch: PropTypes.func,
  confirmLoading: PropTypes.bool,
}

const mapStateToProps = ({ resource, node, loading }) => {
  let n = queryArray(node.list, resource.rightSiderNodeId, 'id')
  return { node: n, confirmLoading: loading.effects['node/operate'] }
}

export default connect(mapStateToProps)(NodeView)
