import React from 'react'
import PropTypes from 'prop-types'
import { Tabs, Form, Input, Row, Col, Modal, Table, Select, message } from 'antd'
import { JsonFormItems } from 'components'
import { queryArray, validateDescriptor as vd, triggerEvent, transformBoolean } from 'utils'
import { connect } from 'dva'
import styles from './RscDetailModal.less'

class RscDetailModal extends React.PureComponent {
  constructor (props) {
    super(props)
    this.state = {
      activeTabKey: '1',
      longdesc: null,
    }
  }
  onOk () {
    let { validateFields, getFieldsValue } = this.props.form
    let { rscTypeLoading } = this.props
    validateFields((errors) => {
      if (errors || rscTypeLoading) {
        this.setState({ activeTabKey: '1' })
        return
      }
      const data = {
        ...getFieldsValue(),
      }
      this.createRsc(data)
    })
  }
  onCancel () {
    let dispatch = this.props.dispatch
    dispatch({ type: 'resource/updateState', payload: { detailModalVisible: false } })
    this.resetFieldsWrapper()
  }
  resetFieldsWrapper () {
    let dispatch = this.props.dispatch
    this.props.form.resetFields()
    this.setState({ longdesc: null, activeTabKey: '1' })
    dispatch({
      type: 'resource/updateState',
      payload: {
        typeDetail: {},
        selectedInstanceAttrs: [],
        selectedMetaAttrs: [],
        selectedActionAttrs: [],
      },
    })
  }
  createRsc (data) {
    let { dispatch, rsc } = this.props
    let isEdit = Boolean(rsc.id)

    if (data.instanceSelect) {
      data.instance_attributes = data.instance_attributes || {}
      // fixbug: attr name contains .
      for (let key in data.instance_attributes) {
        if (key.indexOf('*') > -1) {
          let newKey = key.replace(/\*/g, '.')
          data.instance_attributes[newKey] = data.instance_attributes[key]
          delete data.instance_attributes[key]
        }
      }
    }
    if (data.metaSelect) {
      data.meta_attributes = data.meta_attributes || {}
    }

    let payload = { ...rsc, ...data }

    // 处理用户没有点击到 实例属性 那一页时， instanceSelect没有值的情况
    // 这个时候表单数据data,缺少not required instance attributes
    if (!data.instanceSelect) {
      payload.instance_attributes = { ...rsc.instance_attributes, ...data.instance_attributes }
    }

    if (payload.category === 'primitive') {
      let value = payload.classTypeProvider
      let rscType = {}
      if (value[2]) {
        rscType = {
          class: value[0],
          provider: value[1],
          type: value[2],
        }
      } else {
        rscType = {
          class: value[0],
          type: value[1],
        }
      }
      delete payload.classTypeProvider
      payload = { ...payload, ...rscType }
    }

    delete payload.instanceSelect
    delete payload.metaSelect
    delete payload.actionSelect

    let type = ''
    if (!isEdit) {
      type = 'resource/create'
    } else {
      type = 'resource/update'
    }
    dispatch({ type, payload }).then((resp) => {
      message.success(resp.info, 5)
      this.resetFieldsWrapper()
      dispatch({ type: 'resource/updateState', payload: { detailModalVisible: false } })
      dispatch({ type: 'resource/query' })
    })
  }

  render () {
    let { getFieldDecorator, validateFields } = this.props.form
    let { dispatch, visible, typeDetail, selectedInstanceAttrs, metaAttrs, selectedMetaAttrs, selectedActionAttrs, rsc, types, rsclist, confirmLoading, rscTypesLoading, rscTypeLoading } = this.props
    if (!visible) {
      return null
    }

    let instanceAttrs = typeDetail.parameters
    let isEdit = Boolean(rsc.id)

    const formItemProps = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 9 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 14 },
      },
      style: {
        marginBottom: 12,
      },
    }

    let notRequired = []
    let required = []
    if (instanceAttrs) {
      notRequired = instanceAttrs.filter(param => param.required !== '1')
      required = instanceAttrs.filter(param => param.required === '1')
    }

    let id = {
      label: '资源名称',
      type: 'Input',
      initialValue: rsc.id,
      rules: [vd.required, vd.alphanumeric, vd.startWithAlpha, vd.length(1, 20)],
      props: {
        disabled: isEdit,
        onFocus: () => {
          this.setState({ longdesc: '资源的唯一标识符' })
        },
      },
    }

    const primitiveBasicFormModel = () => {
      // boring data convertion
      let options = []
      const toOption = item => ({ value: item, label: item })

      Object.keys(types).forEach((key) => {
        let item = {}
        item.value = key
        item.label = key
        let value = types[key]
        if (Array.isArray(value)) {
          item.children = value.map(toOption)
        } else {
          let children = []
          Object.keys(value).forEach((k) => {
            let itemInner = {}
            itemInner.value = k
            itemInner.label = k
            let v = value[k]
            if (Array.isArray(v)) {
              itemInner.children = v.map(toOption)
            }
            children.push(itemInner)
          })
          item.children = children
        }
        options.push(item)
      })
      const defaultClassTypeProvider = rsc.provider ? [rsc.class, rsc.provider, rsc.type] : [rsc.class, rsc.type]
      let model = {
        id,
        classTypeProvider: {
          label: '资源类型',
          type: 'Cascader',
          initialValue: isEdit ? defaultClassTypeProvider : undefined,
          props: {
            options,
            expandTrigger: 'hover',
            placeholder: rscTypesLoading ? '加载中...' : '请选择',
            showSearch: true,
            disabled: isEdit || rscTypesLoading,
            onChange: (value) => {
              // 普通资源变更资源类型时，清空实例属性、元属性和操作属性中的值
              dispatch({
                type: 'resource/updateState',
                payload: {
                  rsc: { category: this.props.rsc.category },
                  typeDetail: {},
                  selectedInstanceAttrs: [],
                  selectedMetaAttrs: [],
                  selectedActionAttrs: [],
                },
              })
              this.props.form.setFieldsValue({ metaSelect: [] })
              if (value.length === 0) {
                return
              }
              let rscType = {}
              if (value[2]) {
                rscType = {
                  class: value[0],
                  provider: value[1],
                  type: value[2],
                }
              } else {
                rscType = {
                  class: value[0],
                  type: value[1],
                  provider: undefined,
                }
              }
              dispatch({ type: 'resource/queryRscType', payload: rscType }).then((type) => {
                this.setState({ longdesc: type.longdesc || type.shortdesc })
              })
            },
          },
          rules: [vd.required],
        },
      }

      required.forEach((param) => {
        let old = rsc.instance_attributes && rsc.instance_attributes[param.name]
        let init = old === undefined ? param.content.default : old
        if (param.content.type === 'boolean' && old) {
          old = JSON.parse(old)
        }
        model[`instance_attributes.${param.name}`] = {
          label: param.name,
          type: param.content.type,
          initialValue: init,
          rules: [vd.required],
          props: {
            onFocus: () => {
              this.setState({ longdesc: param.longdesc || param.shortdesc })
            },
          },
          children: param.content.values && param.content.values.map((value) => {
            return { value, label: value }
          }),
        }
      })
      return model
    }

    const groupBasicFormModel = () => {
      let subrscs = rsclist.filter(r => r.type === 'primitive').map((r) => {
        return { value: r.id, label: r.id }
      })
      rsc.rscs && rsc.rscs.forEach((v) => {
        subrscs.push({ value: v, label: v })
      })

      let model = {
        id,
        rscs: {
          label: '组内资源',
          type: 'Select',
          initialValue: rsc.rscs,
          rules: [vd.required],
          children: subrscs,
          props: {
            mode: 'multiple',
            onFocus: () => {
              this.setState({ longdesc: '按照资源优先级顺序选择组内资源' })
            },
            className: styles.groupSelect,
          },
        },
      }
      return model
    }

    const cloneBasicFormModel = () => {
      let model = {
        id,
        rsc_id: {
          label: '克隆对象',
          type: 'Select',
          initialValue: rsc.group ? rsc.group.id : rsc.rsc_id,
          rules: [vd.required],
          children: rsclist.filter(r => r.type !== 'clone').map((r) => {
            return { value: r.id, label: r.id }
          }),
          props: {
            onFocus: () => {
              this.setState({ longdesc: '' })
            },
            disabled: isEdit,
          },
        },
      }
      return model
    }


    const basicAttrTab = () => {
      let model = {}
      switch (rsc.category) {
        case 'primitive':
          model = primitiveBasicFormModel()
          break
        case 'group':
          model = groupBasicFormModel()
          break
        case 'clone':
          model = cloneBasicFormModel()
          break
        default:
          break
      }
      return (
        <Tabs.TabPane tab="基本" key="1">
          <JsonFormItems dataModel={model} getFieldDecorator={getFieldDecorator} formItemProps={formItemProps} />
        </Tabs.TabPane>
      )
    }

    const instanceAttrTab = () => {
      if (rsc.category !== 'primitive' || notRequired.length === 0) {
        return null
      }

      let instanceAttrModel = {
        instanceSelect: {
          label: '需要修改的实例属性',
          type: 'Select',
          initialValue: selectedInstanceAttrs,
          props: {
            showSearch: true,
            mode: 'multiple',
            onChange: (values) => {
              triggerEvent('instanceSelect', 'blur')
              dispatch({ type: 'resource/updateState', payload: { selectedInstanceAttrs: values } })
            },
            onDeselect: () => {
              triggerEvent('instanceSelect', 'blur')
            },
          },
          children: notRequired.map((p) => { return { value: p.name, label: p.name } }),
        },
      }

      notRequired.forEach((param) => {
        let name = param.name
        // fixbug: attr name contains .
        let objectName = name.replace(/\./g, '*')
        if (selectedInstanceAttrs.indexOf(name) === -1) {
          return
        }
        let old = rsc.instance_attributes && rsc.instance_attributes[name]
        if (param.content.type === 'boolean' && old) {
          old = JSON.parse(transformBoolean(old))
        }
        if (param.content.type === 'select') {
          param.content.type = 'string'
        }
        let init = old === undefined ? param.content.default : old

        instanceAttrModel[`instance_attributes.${objectName}`] = {
          label: name,
          initialValue: init,
          type: param.content.type,
          props: {
            onFocus: () => {
              return this.setState({ longdesc: param.longdesc || param.shortdesc })
            },
          },
          children: param.content.values && param.content.values.map((value) => {
            return { value, label: value }
          }),
        }
      })
      return (
        <Tabs.TabPane tab="实例属性" key="2">
          <JsonFormItems dataModel={instanceAttrModel} getFieldDecorator={getFieldDecorator} formItemProps={formItemProps} />
        </Tabs.TabPane>
      )
    }

    const metaAttrTab = () => {
      if (rscTypeLoading || !metaAttrs || Object.keys(metaAttrs).length === 0) {
        return null
      }
      let metaAttrModel = {
        metaSelect: {
          label: '需要修改的元属性',
          type: 'Select',
          initialValue: selectedMetaAttrs,
          props: {
            showSearch: true,
            mode: 'multiple',
            onChange: (values) => {
              triggerEvent('metaSelect', 'blur')
              dispatch({ type: 'resource/updateState', payload: { selectedMetaAttrs: values } })
            },
            onDeselect: () => {
              triggerEvent('metaSelect', 'blur')
            },
          },
          children: Object.keys(metaAttrs).map((key) => { return { value: key, label: key } }),
        },
      }

      Object.keys(metaAttrs).forEach((key) => {
        let param = metaAttrs[key]
        let name = param.name
        if (selectedMetaAttrs.indexOf(name) === -1) {
          return
        }

        let old = rsc.meta_attributes && rsc.meta_attributes[name]
        if (param.content.type === 'boolean' && old) {
          old = JSON.parse(old)
        }
        let init = old === undefined ? param.content.default : old
        metaAttrModel[`meta_attributes.${name}`] = {
          label: name,
          initialValue: init,
          type: param.content.type,
          props: {
            onFocus: () => {
              this.setState({ longdesc: param.content && param.content.dec })
            },
          },
          children: param.content.values && param.content.values.map((value) => {
            return { value, label: value }
          }),
        }
      })
      return (
        <Tabs.TabPane tab="元属性" key="3">
          <JsonFormItems dataModel={metaAttrModel} getFieldDecorator={getFieldDecorator} formItemProps={formItemProps} />
        </Tabs.TabPane>
      )
    }

    const renderInputColumns = (text, record, column) => {
      return (<Input style={{ margin: '-5px 0' }}
        value={text}
        onChange={(e) => {
          let actions = rsc.actions.map((item) => {
            if (item.name === record.name) {
              item[column] = e.target.value
            }
            return item
          })
          dispatch({ type: 'resource/updateState', payload: { rsc: { ...rsc, actions } } })
        }}
      />)
    }

    const renderSelectColumns = (text, record, column, children) => {
      return (<Select style={{ margin: '-5px 0' }}
        value={text}
        onChange={(value) => {
          let actions = rsc.actions.map((item) => {
            if (item.name === record.name) {
              item[column] = value
            }
            return item
          })
          dispatch({ type: 'resource/updateState', payload: { rsc: { ...rsc, actions } } })
        }}
      >
        {
          children.map(c =>
            <Select.Option key={c} value={c}>{c}</Select.Option>
          )
        }
      </Select>)
    }

    const actionAttrTab = () => {
      if (rsc.category !== 'primitive' || !typeDetail.actions) {
        return undefined
      }

      let actionAttrModel = {
        actionSelect: {
          label: '需要修改的操作属性',
          type: 'Select',
          initialValue: selectedActionAttrs,
          props: {
            showSearch: true,
            mode: 'multiple',
            onChange: (values) => {
              let actions = values.map((value) => {
                let action = queryArray(rsc.actions, value, 'name')
                return action || queryArray(typeDetail.actions, value, 'name')
              })
              triggerEvent('actionSelect', 'blur')
              dispatch({ type: 'resource/updateState', payload: { selectedActionAttrs: values, rsc: { ...rsc, actions } } })
            },
            onDeselect: () => {
              triggerEvent('actionSelect', 'blur')
            },
          },
          children: typeDetail.actions.map((action) => { return { value: action.name, label: action.name } }),
        },
      }

      let columns = [{
        title: 'name',
        dataIndex: 'name',
        width: '14%',
      }, {
        title: 'interval',
        dataIndex: 'interval',
        width: '10%',
        render: (text, record) => renderInputColumns(text, record, 'interval'),
      }, {
        title: 'start-delay',
        dataIndex: 'start-delay',
        width: '10%',
        render: (text, record) => renderInputColumns(text, record, 'start-delay'),
      }, {
        title: 'timeout',
        dataIndex: 'timeout',
        width: '10%',
        render: (text, record) => renderInputColumns(text, record, 'timeout'),
      }, {
        title: 'role',
        dataIndex: 'role',
        width: '15%',
        render: (text, record) => renderSelectColumns(text, record, 'role', ['Stopped', 'Started', 'Master', 'Slave']),
      }, {
        title: 'requires',
        dataIndex: 'requires',
        width: '15%',
        render: (text, record) => renderSelectColumns(text, record, 'requires', ['nothing', 'quorum', 'fencing']),
      }, {
        title: 'on-fail',
        dataIndex: 'on-fail',
        width: '16%',
        render: (text, record) => renderSelectColumns(text, record, 'on-fail', ['ignore', 'block', 'stop', 'restart', 'standby', 'fence']),
      }]

      return (
        <Tabs.TabPane tab="操作属性" key="4">
          <JsonFormItems dataModel={actionAttrModel} getFieldDecorator={getFieldDecorator} formItemProps={formItemProps} />
          <Table bordered dataSource={rsc.actions} columns={columns} rowKey="name" pagination={false} locale={{ emptyText: '暂无数据' }} />
        </Tabs.TabPane>
      )
    }
    let modalProps = {
      visible,
      onOk: this.onOk.bind(this),
      onCancel: this.onCancel.bind(this),
      width: 700,
      title: rsc.id ? '编辑资源' : '创建资源',
      maskClosable: false,
      style: {
        top: 47,
      },
      confirmLoading,
    }

    const onChange = (activeKey) => {
      validateFields((errors) => {
        if (!errors) {
          this.setState({ activeTabKey: activeKey, longdesc: null })
        }
      })
    }

    return (
      <Modal {...modalProps} >
        <Row style={{ minHeight: 200 }}>
          <Col span={this.state.activeTabKey !== '4' ? 15 : 24}>
            <Form >
              <Tabs defaultActiveKey="1" size="small" activeKey={this.state.activeTabKey} onChange={onChange}>
                {basicAttrTab()}
                {instanceAttrTab()}
                {metaAttrTab()}
                {actionAttrTab()}
              </Tabs>
            </Form>
          </Col>

          {
            this.state.longdesc &&
            <Col span={9} className={styles.desc}>
              <p> {this.state.longdesc} </p>
            </Col>
          }

        </Row>
      </Modal>
    )
  }
}

RscDetailModal.propTypes = {
  dispatch: PropTypes.func,
  types: PropTypes.object,
  typeDetail: PropTypes.object,
  selectedInstanceAttrs: PropTypes.array,
  metaAttrs: PropTypes.object,
  selectedMetaAttrs: PropTypes.array,
  selectedActionAttrs: PropTypes.array,
  rsc: PropTypes.object,
  visible: PropTypes.bool,
  rsclist: PropTypes.array,
  confirmLoading: PropTypes.bool,
  rscTypesLoading: PropTypes.bool,
  rscTypeLoading: PropTypes.bool,
}

const mapStateToProps = ({ resource, loading }) => {
  if (!resource.detailModalVisible) {
    return {}
  }
  return {
    types: resource.types,
    typeDetail: resource.typeDetail,
    selectedInstanceAttrs: resource.selectedInstanceAttrs,
    metaAttrs: resource.metaAttrs,
    selectedMetaAttrs: resource.selectedMetaAttrs,
    selectedActionAttrs: resource.selectedActionAttrs,
    rsc: resource.rsc,
    visible: resource.detailModalVisible,
    rsclist: resource.list,
    confirmLoading: resource.rsc.id ? loading.effects['resource/update'] : loading.effects['resource/create'],
    rscTypesLoading: loading.effects['resource/queryRscTypes'],
    rscTypeLoading: loading.effects['resource/queryRscType'],
  }
}

export default connect(mapStateToProps)(Form.create()(RscDetailModal))
