import React from 'react'
import PropTypes from 'prop-types'
import { validateDescriptor as vd, handleOperation } from 'utils'
import { connect } from 'dva'
import { JsonFormModal } from '../JsonForm'

class RscMigrationModal extends React.PureComponent {
  onOk (data, resetFields) {
    let dispatch = this.props.dispatch
    let period = parseInt(data.period, 10)
    if (!isNaN(period)) {
      data.period = `PYMDT${String(period)}HMS`
    } else {
      delete data.period
    }
    delete data.id
    handleOperation(dispatch, 'resource', this.props.rsc.id, 'migrate', data).then(() => {
      dispatch({ type: 'resource/updateState', payload: { rscMigrationModalVisible: false } })
      resetFields()
    })
  }
  onCancel () {
    let dispatch = this.props.dispatch
    dispatch({ type: 'resource/updateState', payload: { rscMigrationModalVisible: false } })
  }
  render () {
    let { visible, nodelist, rsc, confirmLoading, runningNode } = this.props

    if (!visible) {
      return null
    }

    let dataModel = {
      id: {
        label: '资源名称',
        type: <span> {rsc.id} </span>,
      },
      to_node: {
        label: '迁移至节点',
        type: 'Select',
        children: nodelist.filter(node => runningNode.indexOf(node.id) === -1).map((node) => { return { label: node.id, value: node.id } }),
        rules: [vd.required],
      },
      is_force: {
        label: '强制迁移',
        type: 'Switch',
        initialValue: false,
      },
      period: {
        label: '有效期',
        type: 'Input',
        props: {
          addonAfter: '小时',
        },
        rules: [{
          pattern: /^\d*$/, message: '请输入0或正整数',
        }],
      },
    }
    let modalProps = {
      visible,
      onOk: this.onOk.bind(this),
      onCancel: this.onCancel.bind(this),
      dataModel,
      maskClosable: false,
      title: '迁移资源',
      formItemProps: {
        labelCol: {
          span: 9,
        },
        wrapperCol: {
          span: 11,
        },
      },
      confirmLoading,
    }
    return (
      <JsonFormModal {...modalProps} />
    )
  }
}

RscMigrationModal.propTypes = {
  visible: PropTypes.bool,
  runningNode: PropTypes.array,
  nodelist: PropTypes.array,
  rsc: PropTypes.object,
  confirmLoading: PropTypes.bool,
  dispatch: PropTypes.func,
}

const mapStateToProps = ({ resource, node, loading }) => {
  if (!resource.rscMigrationModalVisible) {
    return {}
  }
  return {
    visible: resource.rscMigrationModalVisible,
    runningNode: resource.selectedRows[0].running_node || [],
    nodelist: node.list,
    rsc: resource.selectedRows[0],
    confirmLoading: loading.effects['resource/operate'],
  }
}

export default connect(mapStateToProps)(RscMigrationModal)
