import React, { PureComponent } from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { message } from 'antd'
import { triggerEvent } from 'utils'
import { JsonFormModal } from '../JsonForm'

/* this.props.dispatch */
class RscRestrictions extends PureComponent {
  getSelectChildrenData (level) {
    let { restrictionModel, assistModel, modalTitle } = this.props
    let item
    let otherLevelNodes = []
    for (item in restrictionModel) {
      if (item !== level) {
        otherLevelNodes = otherLevelNodes.concat(restrictionModel[item])
      }
    }

    return assistModel
      .filter((node) => {
        return modalTitle === 'location' ?
          otherLevelNodes.indexOf(node.id) < 0 :
          otherLevelNodes.indexOf(node.id) < 0 && node.type !== 'clone' && node.id !== restrictionModel.rsc_id
      }).map((node) => {
        return {
          label: node.id,
          value: node.id,
        }
      })
  }

  getFormSelectItemDataModel (property, label) {
    let { restrictionModel, dispatch } = this.props
    this.formDataModel[property] = {
      label,
      type: 'Select',
      props: {
        mode: 'multiple',
        placeholder: '请选择',
        onChange: (value) => {
          // 不能用对象的引用
          let restrictionsLocal = JSON.parse(JSON.stringify(restrictionModel))
          restrictionsLocal[property] = value
          triggerEvent(`${property}`, 'blur')
          dispatch({ type: 'resource/updateState', payload: { restrictions: restrictionsLocal } })
        },
        onDeselect: () => {
          triggerEvent(`${property}`, 'blur')
        },
      },
      children: this.getSelectChildrenData(property),
      initialValue: restrictionModel[property] ? restrictionModel[property] : [],
    }
  }

  initModalTitleAndDataModel () {
    let { restrictionModel, modalTitle, assistModel } = this.props
    this.title = ''
    this.formDataModel = {}

    this.formDataModel.rsc_id = {
      label: '资源名称',
      initialValue: restrictionModel.rsc_id,
      type: <span className="ant-form-text">{restrictionModel.rsc_id}</span>,
    }

    switch (modalTitle) {
      case 'location':
        this.title = '资源位置'
        for (let i = 0; i < assistModel.length; i += 1) {
          if (i === 0) {
            this.getFormSelectItemDataModel('Master Node', 'Master Node')
          } else {
            let levelTag = i
            this.getFormSelectItemDataModel(`Slave ${levelTag}`, `Slave ${levelTag}`)
          }
        }

        break
      case 'colocation':
        this.title = '资源协同'
        this.getFormSelectItemDataModel('same_node', '同节点资源')
        this.getFormSelectItemDataModel('diff_node', '互斥节点资源')
        break
      default:
        this.title = '资源顺序'
        this.getFormSelectItemDataModel('before_rscs', '前置资源')
        this.getFormSelectItemDataModel('after_rscs', '后置资源')
    }
  }
  render () {
    let { restrictionModel, modalVisible, modalTitle, dispatch, selectedRowKey, confirmLoading } = this.props
    if (!modalVisible || !restrictionModel) {
      return null
    }

    this.initModalTitleAndDataModel()

    const modalProps = {
      visible: modalVisible,
      onOk: (data, resetFields) => {
        if (modalTitle === 'location') {
          data.node_level = []
          for (let prop of Object.keys(data)) {
            if (prop === 'rsc_id' || prop === 'node_level') {
              continue
            }
            data[prop].forEach((node) => {
              data.node_level.push({
                level: prop,
                node,
              })
            })
          }
        }
        dispatch({ type: 'resource/operate', payload: { id: selectedRowKey, action: modalTitle, ...data } }).then((resp) => {
          message.success(resp.info, 5)
          resetFields()
          dispatch({ type: 'resource/updateState', payload: { rscRestrictionModalVisible: false } })
          dispatch({ type: 'resource/query' })
        })
      },
      onCancel: () => {
        dispatch({ type: 'resource/updateState', payload: { rscRestrictionModalVisible: false } })
      },
      dataModel: this.formDataModel,
      maskClosable: false,
      title: this.title,
      formItemProps: {
        labelCol: {
          span: 9,
        },
        wrapperCol: {
          span: 11,
        },
      },
      confirmLoading,
    }
    return (
      <JsonFormModal {...modalProps} />
    )
  }
}

RscRestrictions.propTypes = {
  dispatch: PropTypes.func,
  modalVisible: PropTypes.bool,
  confirmLoading: PropTypes.bool,
  assistModel: PropTypes.array,
  modalTitle: PropTypes.string,
  restrictionModel: PropTypes.object,
  selectedRowKey: PropTypes.string,
}

const mapStateToProps = ({ node, resource, loading }) => {
  const modalTitle = resource.rscRestrictionModalTitle
  if (!resource.rscRestrictionModalVisible) {
    return {}
  }
  return {
    assistModel: modalTitle === 'location' ? node.list : resource.list,
    modalTitle: resource.rscRestrictionModalTitle,
    restrictionModel: resource.restrictions,
    modalVisible: resource.rscRestrictionModalVisible,
    selectedRowKey: resource.selectedRows[0].id,
    confirmLoading: loading.effects['resource/operate'],
  }
}
export default connect(mapStateToProps)(RscRestrictions)
