import React from 'react'
import PropTypes from 'prop-types'
import { connect } from 'dva'
import { Modal, Row, Col, Select, message } from 'antd'
import { triggerTimedTask } from 'utils'
import styles from './SettingModal.less'

const Option = Select.Option
class SettingModal extends React.PureComponent {
  constructor (props) {
    super(props)
    this.refreshType = null
    this.handleCancel = this.handleCancel.bind(this)
    this.handleOk = this.handleOk.bind(this)
    this.handleChange = this.handleChange.bind(this)
  }

  handleChange (value) {
    this.refreshType = value
  }

  handleOk () {
    const { autoReFreshTaskId, autoReFreshId, dispatch } = this.props
    let selectedRefreshType = this.refreshType
    let id = 0

    if (selectedRefreshType === null || autoReFreshId === selectedRefreshType) {
      this.closeModal()
      return
    }

    clearInterval(autoReFreshTaskId)
    if (selectedRefreshType === '0') {
      dispatch({ type: 'app/switchAutoReFreshState', payload: { autoReFreshId: selectedRefreshType, autoReFreshTaskId: id } })
      this.closeModal()
      return
    }

    id = triggerTimedTask(dispatch, selectedRefreshType)

    dispatch({ type: 'app/switchAutoReFreshState', payload: { autoReFreshId: selectedRefreshType, autoReFreshTaskId: id } })

    this.closeModal()
  }

  closeModal () {
    this.handleCancel()
    message.success('设置成功', 5)
  }

  handleCancel () {
    this.props.dispatch({ type: 'app/updateState', payload: { SettingModalVisible: false } })
  }

  render () {
    if (!this.props.modalVisible) {
      return null
    }

    const modalProps = {
      visible: this.props.modalVisible,
      maskClosable: false,
      title: '刷新设置',
      style: { top: 47 },
      onCancel: this.handleCancel,
      onOk: this.handleOk,
    }


    return (
      <Modal {...modalProps}>
        <Row className={styles.rowline}>
          <Col span={6} key="settingLable" className={styles.titlealign}>刷新设置:</Col>
          <Col span={14} key="settingSelect">
            <Select defaultValue={this.props.autoReFreshId} className={styles.selectwitdh} onChange={this.handleChange}>
              <Option value="0">不自动刷新</Option>
              <Option value="1">每5秒刷新</Option>
              <Option value="2">每10秒刷新</Option>
            </Select>
          </Col>
        </Row>
      </Modal>
    )
  }
}

SettingModal.propTypes = {
  modalVisible: PropTypes.bool,
  autoReFreshId: PropTypes.string,
  autoReFreshTaskId: PropTypes.number,
  dispatch: PropTypes.func,
}

const mapStateToProps = ({ app }) => {
  if (!app.SettingModalVisible) {
    return {}
  }
  return {
    modalVisible: app.SettingModalVisible,
    autoReFreshId: app.autoReFreshId,
    autoReFreshTaskId: app.autoReFreshTaskId,
  }
}

export default connect(mapStateToProps)(SettingModal)
