/* global window, document */
import { parse } from 'qs'
import config from 'config'
import { logout, registerLicense, getLogs } from 'services/app'

const { prefix } = config

export default {
  namespace: 'app',
  state: {
    menuPopoverVisible: false,
    siderFold: window.localStorage.getItem(`${prefix}siderFold`) === 'true',
    isNavbar: document.body.clientWidth < 769,
    locationPathname: '',
    locationQuery: {},

    licenseModalVisible: false,

    SettingModalVisible: false,
    autoReFreshId: !window.localStorage.getItem(`${prefix}autoReFreshId`) ? '0' : window.localStorage.getItem(`${prefix}autoReFreshId`),
    autoReFreshTaskId: 0,
  },
  subscriptions: {
    setup ({ dispatch }) {
      let tid

      window.onresize = () => {
        clearTimeout(tid)
        tid = setTimeout(() => {
          dispatch({ type: 'changeNavbar' })
        }, 300)
      }
    },

  },
  effects: {
    * logout ({
      payload,
    }, { call }) {
      yield call(logout, parse(payload))
      window.location = '/login'
    },

    * changeNavbar (action, { put, select }) {
      const { app } = yield (select(_ => _))
      const isNavbar = document.body.clientWidth < 769
      if (isNavbar !== app.isNavbar) {
        yield put({ type: 'handleNavbar', payload: isNavbar })
      }
    },

    * registerLicense ({
      payload,
    }, { call }) {
      return yield call(registerLicense, parse(payload))
    },

    * getLogs ({ payload }, { call }) {
      return yield call(getLogs, parse(payload))
    },
  },
  reducers: {
    updateState (state, { payload }) {
      return {
        ...state,
        ...payload,
      }
    },

    switchSider (state) {
      window.localStorage.setItem(`${prefix}siderFold`, !state.siderFold)
      return {
        ...state,
        siderFold: !state.siderFold,
      }
    },

    switchMenuPopver (state) {
      return {
        ...state,
        menuPopoverVisible: !state.menuPopoverVisible,
      }
    },

    handleNavbar (state, { payload }) {
      return {
        ...state,
        isNavbar: payload,
      }
    },

    switchAutoReFreshState (state, { payload }) {
      window.localStorage.setItem(`${prefix}autoReFreshId`, payload.autoReFreshId)
      return {
        ...state,
        autoReFreshId: payload.autoReFreshId,
        autoReFreshTaskId: payload.autoReFreshTaskId,
      }
    },
  },
}
