import { parse } from 'qs'
import modelExtend from 'dva-model-extend'
import { query, update, queryAlarm, updateAlarm, queryHeartBeat, updateHeartBeat, queryDevices, queryDrbd, updateDrbd, deleteDrbd, queryGfs, updateGfs, deleteGfs, createScript, queryHeartBeatStatus, queryCommands, operateCommand } from 'services/cluster'
import { model } from 'models/common'
import { transformData } from 'utils'

export default modelExtend(model, {
  namespace: 'cluster',
  state: {
    clusterSettingModel: null,
    clusterSettingModalVisible: false,

    alarmSettingModel: null,
    alarmSettingModalVisible: false,

    heartBeatModel: null,
    devices: [],
    heartBeatModalVisible: false,

    drbdModel: null,
    drbdModalVisible: false,

    gfsModel: null,
    gfsModalVisible: false,

    hbstatus: 0,

    scriptModalVisible: false,

    commandModalVisible: false,
    commands: null,
  },
  subscriptions: {

  },
  effects: {
    * query ({
      payload,
    }, { call, put }) {
      const resp = yield call(query, parse(payload))
      transformData(resp.data.parameters)
      yield put({
        type: 'updateState',
        payload: {
          clusterSettingModel: resp.data.parameters,
        },
      })
    },
    * update ({
      payload,
    }, { call }) {
      return yield call(update, parse(payload))
    },
    * queryAlarm ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryAlarm, parse(payload))
      yield put({
        type: 'updateState',
        payload: {
          alarmSettingModel: resp.data,
        },
      })
    },
    * updateAlarm ({
      payload,
    }, { call }) {
      return yield call(updateAlarm, parse(payload))
    },
    * queryHeartBeat ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryHeartBeat, parse(payload))
      const heartBeatModel = []

      const getNhbObj = (heartBeat) => {
        let hbType = ''

        if (heartBeat.mcastaddr === undefined) {
          hbType = 'uc'
        } else if (heartBeat.mcastaddr === 'broadcast') {
          hbType = 'bc'
        } else {
          hbType = 'mc'
        }

        return {
          type: hbType,
          flag: true,
          ...heartBeat,
          mcastaddr: heartBeat.mcastaddr === 'broadcast' ? '' : heartBeat.mcastaddr,
        }
      }

      if (resp.data[0]) { // 排除配置文件为空的情况
        resp.data.forEach((heartBeat) => {
          if (heartBeat.ringnumber === 0) { // 主网络心跳
            heartBeatModel[0] = getNhbObj(heartBeat)
          } else if (heartBeat.mcastport) { // 备用网络心跳
            heartBeatModel[1] = getNhbObj(heartBeat)
          } else {
            heartBeatModel[2] = {// 磁盘心跳
              flag: true,
              ...heartBeat,
            }
          }
        })
      }
      yield put({
        type: 'updateState',
        payload: { heartBeatModel },
      })
    },
    * updateHeartBeat ({
      payload,
    }, { call }) {
      return yield call(updateHeartBeat, parse(payload))
    },
    * queryDevices ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryDevices, parse(payload))
      const devices = []

      resp.data.forEach((device) => {
        devices.push({
          label: device,
          value: device,
        })
      })

      yield put({
        type: 'updateState',
        payload: { devices },
      })
    },
    * queryDrbd ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryDrbd, parse(payload))
      let drbdModel = {
        flag: false,
        drbd_type: '0',
      }
      // 未启用drbd时，接口返回空对象
      if (JSON.stringify(resp.data) !== '{}') {
        drbdModel.flag = true
        Object.assign(drbdModel, resp.data)
      }

      yield put({
        type: 'updateState',
        payload: { drbdModel },
      })
    },
    * updateDrbd ({
      payload,
    }, { call }) {
      return yield call(updateDrbd, parse(payload))
    },
    * deleteDrbd (_, { call }) {
      return yield call(deleteDrbd)
    },
    * queryGfs ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryGfs, parse(payload))
      let enabled = (resp.data && Object.keys(resp.data).length !== 0)

      yield put({
        type: 'updateState',
        payload: {
          gfsModel: {
            enabled,
            ...resp.data,
          },
        },
      })
    },
    * updateGfs ({
      payload,
    }, { call }) {
      return yield call(updateGfs, parse(payload))
    },
    * deleteGfs (_, { call }) {
      return yield call(deleteGfs)
    },
    * createScript ({
      payload,
    }, { call }) {
      return yield call(createScript, parse(payload))
    },
    * queryHeartBeatStatus ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryHeartBeatStatus, parse(payload))

      yield put({
        type: 'updateState',
        payload: {
          hbstatus: resp.data,
        },
      })
    },
    * queryCommands ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryCommands, parse(payload))

      yield put({
        type: 'updateState',
        payload: {
          commands: resp.data,
        },
      })
    },

    * operateCommand ({
      payload,
    }, { call }) {
      return yield call(operateCommand, parse(payload))
    },
  },
  reducers: {
  },
})
