/* global window */
import { parse } from 'qs'
import config from 'config'
import modelExtend from 'dva-model-extend'
import { query, detail, update, create, operate, queryRscTypes, queryRscType, queryRscAttrs, queryRscRestriction, queryDrbdStatus } from 'services/resource'
import { model } from 'models/common'
import { transformData, triggerTimedTask, getDrbdEnable } from 'utils'

const { prefix } = config

export default modelExtend(model, {
  namespace: 'resource',
  state: {
    list: [],
    rightSiderFold: true,
    rightSiderNodeId: '',
    selectedRowKeys: [],
    selectedRows: [],

    detailModalVisible: false,
    types: {},
    typeDetail: {},
    selectedInstanceAttrs: [],
    metaAttrs: {},
    selectedMetaAttrs: [],
    selectedActionAttrs: [],
    rsc: {},

    rscRestrictionModalVisible: false,
    restrictions: null,
    rscRestrictionModalTitle: '',

    rscMigrationModalVisible: false,

    mainLockVisible: false,
  },
  subscriptions: {
    setup ({ dispatch, history }) {
      history.listen(({ pathname }) => {
        if (pathname === '/resource' || pathname === '/') {
          dispatch({ type: 'firstQuery' }).then(() => {
            let autoReFreshId = !window.localStorage.getItem(`${prefix}autoReFreshId`) ? '0' : window.localStorage.getItem(`${prefix}autoReFreshId`)
            let autoReFreshTaskId = 0
            dispatch({ type: 'cluster/queryHeartBeatStatus' })
            if (autoReFreshId !== '0') {
              autoReFreshTaskId = triggerTimedTask(dispatch, autoReFreshId)
              dispatch({ type: 'app/switchAutoReFreshState', payload: { autoReFreshId, autoReFreshTaskId } })
            }
          }).catch((resp) => {
            if (resp.message !== '请检查HA集群是否处于启动状态！') {
              return
            }
            dispatch({
              type: 'updateState',
              payload: {
                mainLockVisible: true,
              },
            })
          })
        }
      })
    },
  },
  effects: {
    // FIXME: duplicated function 'firstQuery' and 'query' to handle loader problem
    // On auto refresh, we don't want to activate loader
    // On page entry, we want to activate loader
    * firstQuery ({
      payload,
    }, { call, put }) {
      const resp = yield call(query, parse(payload))
      let drbdEnable = getDrbdEnable(resp.data)
      if (drbdEnable) {
        yield put({
          type: 'queryDrbdStatus',
        })
      }
      yield put({
        type: 'updateState',
        payload: {
          rawlist: resp.data,
        },
      })
      yield put({
        type: 'preprocessingState',
      })
    },
    * query ({
      payload,
    }, { call, put }) {
      try {
        const resp = yield call(query, parse(payload))
        let drbdEnable = getDrbdEnable(resp.data)
        if (drbdEnable) {
          yield put({
            type: 'queryDrbdStatus',
          })
        }
        yield put({
          type: 'updateState',
          payload: {
            rawlist: resp.data,
            mainLockVisible: false,
          },
        })
        yield put({
          type: 'preprocessingState',
        })
      } catch (err) {
        if (err.message === '请检查HA集群是否处于启动状态！') {
          yield put({
            type: 'updateState',
            payload: {
              mainLockVisible: true,
            },
          })
        }
      }
    },
    * create ({
      payload,
    }, { call }) {
      return yield call(create, parse(payload))
    },
    * detail ({
      payload,
    }, { call, put, select, take }) {
      const resp = yield call(detail, parse(payload))

      let rsc = resp.data
      yield put({
        type: 'updateState',
        payload: {
          rsc,
        },
      })

      if (rsc.meta_attributes) {
        let attrs = []
        Object.keys(rsc.meta_attributes).forEach((key) => {
          attrs.push(key)
        })

        yield put({
          type: 'updateState',
          payload: {
            selectedMetaAttrs: attrs,
          },
        })
      }

      if (rsc.category !== 'primitive') {
        return Promise.resolve()
      }

      yield put({
        type: 'queryRscType',
        payload: {
          class: rsc.class,
          provider: rsc.provider,
          type: rsc.type,
        },
      })

      yield take('queryRscType/@@end')
      let { typeDetail } = yield select(_ => _.resource)
      // Collect not-required-parameters
      if (rsc.instance_attributes) {
        let attrs = []
        typeDetail.parameters && typeDetail.parameters.forEach((param) => {
          if (param.required !== '1' && rsc.instance_attributes[param.name]) {
            attrs.push(param.name)
          }
        })

        yield put({
          type: 'updateState',
          payload: {
            selectedInstanceAttrs: attrs,
          },
        })
      }

      if (rsc.actions) {
        let attrs = []
        rsc.actions.forEach((action) => {
          attrs.push(action.name)
        })

        yield put({
          type: 'updateState',
          payload: {
            selectedActionAttrs: attrs,
          },
        })
      }

      return Promise.resolve()
    },
    * update ({
      payload,
    }, { call }) {
      return yield call(update, parse(payload))
    },
    * operate ({
      payload,
    }, { call }) {
      return yield call(operate, parse(payload))
    },
    * queryRscTypes ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryRscTypes, parse(payload))
      yield put({
        type: 'updateState',
        payload: {
          types: resp.data,
        },
      })
    },
    * queryRscType ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryRscType, parse(payload))
      transformData(resp.data.parameters)
      yield put({
        type: 'updateState',
        payload: {
          typeDetail: resp.data,
        },
      })
      return Promise.resolve(resp.data)
    },
    * queryRscAttrs ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryRscAttrs, parse(payload))
      transformData(resp.data)
      yield put({
        type: 'updateState',
        payload: {
          metaAttrs: resp.data,
        },
      })
    },
    * queryRscRestriction ({
      payload,
    }, { call, put }) {
      const resp = yield call(queryRscRestriction, parse(payload))

      if (payload.relation === 'location') {
        resp.data.node_level.forEach((location) => {
          if (!resp.data[location.level]) {
            resp.data[location.level] = []
          }

          resp.data[location.level].push(location.node)
        })
      }
      yield put({
        type: 'updateState',
        payload: {
          restrictions: resp.data,
        },
      })
    },
    * queryDrbdStatus (_, { call, put }) {
      const resp = yield call(queryDrbdStatus)

      yield put({
        type: 'updateListWithDrbdStatus',
        payload: {
          drbdStatus: resp.data.code, // '0'->正常； '1'-> 异常
          drbdMsg: resp.data.message,
        },
      })
    },
  },
  reducers: {
    clearSelectedRows (state) {
      let payload = {
        selectedRowKeys: [],
        selectedRows: [],
      }
      return {
        ...state,
        ...payload,
      }
    },
    preprocessingState (state) {
      let selectedRows = []
      let selectedRowKeys = state.selectedRowKeys
      let selected = state.selectedRows[0]
      state.list = state.rawlist
      state.list.forEach((rsc) => {
        if (selected && selected.id === rsc.id) {
          selectedRows = [rsc]
        }
        rsc.key = rsc.id
        rsc.children = rsc.subrscs
        rsc.children && rsc.children.forEach((child) => {
          if (selected && selected.id === child.id) {
            selectedRows = [child]
          }
          child.isChild = true
          child.children = child.subrscs
          child.key = `${child.id}-${rsc.id}`
          child.children && child.children.forEach((grandson) => {
            if (selected && selected.id === grandson.id) {
              selectedRows = [grandson]
            }
            grandson.isChild = true
            grandson.key = `${grandson.id}-${child.id}-${rsc.id}`
          })
        })
      })

      if (selectedRows.length === 0) {
        selectedRowKeys = []
      }
      return {
        ...state,
        selectedRows,
        selectedRowKeys,
      }
    },
    updateListWithDrbdStatus (state, { payload }) {
      let { drbdStatus, drbdMsg } = payload
      state.list.forEach((rsc) => {
        if (drbdStatus !== '0' && (rsc.id === 'group-fs-ps' || rsc.id === 'ms-drbd')) {
          rsc.drbdStatus = drbdStatus
          rsc.drbdMsg = drbdMsg
        }
      })

      return { ...state }
    },
  },
})
